from typing import List

from cidc_schemas.prism.constants import SUPPORTED_MANIFESTS

from .utils import (
    copy_dict_with_branch,
    get_prismify_args,
    get_test_trial,
    LocalFileUploadEntry,
    PrismTestData,
)

manifest_data_generators = []


def manifest_data_generator(f):
    manifest_data_generators.append(f)
    return f


@manifest_data_generator
def plasma() -> PrismTestData:
    upload_type = "plasma"
    prismify_args = get_prismify_args(upload_type)
    prismify_patch = {
        "protocol_identifier": "test_prism_trial_id",
        "shipments": [
            {
                "manifest_id": "test_prism_trial_id_PLASMA",
                "assay_priority": "4",
                "assay_type": "Olink",
                "receiving_party": "MSSM_Gnjatic",
                "courier": "USPS",
                "tracking_number": "TrackN",
                "shipping_condition": "Frozen Dry Ice",
                "date_shipped": "2001-10-10 00:00:00",
                "date_received": "2002-10-10 00:00:00",
                "quality_of_shipment": "Normal",
                "ship_from_sender_person_name": "George",
                "ship_from_sender_email": "test@nih.gov",
                "ship_from_sender_phone": "111-1111",
                "ship_from_name": "Jared",
                "ship_from_street1": "abc",
                "ship_from_street2": "abc",
                "ship_from_city": "seattle",
                "ship_from_state": "WA",
                "ship_from_zip": "98003",
                "ship_from_country": "USA",
                "ship_to_name": "Marie",
                "ship_to_street1": "abc",
                "ship_to_street2": "abc",
                "ship_to_city": "Plano",
                "ship_to_state": "TX",
                "ship_to_zip": "99013",
                "ship_to_country": "USA",
            }
        ],
        "participants": [
            {
                "samples": [
                    {
                        "shipping_entry_number": 1,
                        "cimac_id": "CTTTP01A1.00",
                        "surgical_pathology_report_id": "test_encrypted('Surgical pathology report 1')",
                        "clinical_report_id": "test_encrypted('clinical report 1')",
                        "collection_event_name": "Baseline",
                        "diagnosis_verification": "Local pathology review was not consistent",
                        "site_description": "ADRENAL GLANDS",
                        "topography_code": "C00.2",
                        "topography_description": "External lower lip",
                        "histology_behavior": "8003/3",
                        "histology_behavior_description": "Neoplasm, malignant",
                        "parent_sample_id": "test_encrypted('TTTPP1A1')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1A1')",
                        "processed_sample_quantity": 5,
                        "box_number": "1",
                        "sample_location": "A1",
                        "type_of_sample": "Whole Blood",
                        "sample_collection_procedure": "Core Biopsy",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_volume": 1.0,
                        "processed_sample_volume_units": "Other",
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Sample Returned",
                        "comments": "Comment",
                    },
                    {
                        "shipping_entry_number": 2,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TTTPP1A2')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1A2')",
                        "cimac_id": "CTTTP01A2.00",
                        "box_number": "1",
                        "sample_location": "A2",
                        "type_of_sample": "Whole Blood",
                        "sample_collection_procedure": "Core Biopsy",
                        "type_of_primary_container": "Fecal Collection Container with NA Stabilizer",
                        "type_of_primary_container_other": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_volume": 2.0,
                        "processed_sample_volume_units": "Other",
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Sample Returned",
                        "comments": "No comment",
                    },
                    {
                        "shipping_entry_number": 3,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TTTPP1A3')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1A3')",
                        "cimac_id": "CTTTP01A3.00",
                        "box_number": "1",
                        "sample_location": "A3",
                        "type_of_sample": "Whole Blood",
                        "sample_collection_procedure": "Other",
                        "sample_collection_procedure_other": "Other form of collection",
                        "type_of_primary_container": "Fecal Collection Container with NA Stabilizer",
                        "type_of_primary_container_other": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_volume": 3.0,
                        "processed_sample_volume_units": "Other",
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Sample Returned",
                    },
                ],
                "participant_id": "test_encrypted('TTTP01')",
                "cimac_participant_id": "CTTTP01",
                "gender": "Female",
                "race": "Asian",
                "ethnicity": "Hispanic or Latino",
                "cohort_name": "Arm_Z",
            },
            {
                "samples": [
                    {
                        "shipping_entry_number": 4,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TTTPP2A1')",
                        "processed_sample_id": "test_encrypted('BIOBANK 2A1')",
                        "cimac_id": "CTTTP02A1.00",
                        "box_number": "1",
                        "sample_location": "A4",
                        "type_of_sample": "Whole Blood",
                        "sample_collection_procedure": "Core Biopsy",
                        "type_of_primary_container": "Fecal Collection Container with NA Stabilizer",
                        "type_of_primary_container_other": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_volume": 4.0,
                        "processed_sample_volume_units": "Other",
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Sample Returned",
                    },
                    {
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TTTPP2A2')",
                        "processed_sample_id": "test_encrypted('BIOBANK 2A2')",
                        "cimac_id": "CTTTP02A2.00",
                        "box_number": "1",
                        "sample_location": "A5",
                        "type_of_sample": "Whole Blood",
                        "sample_collection_procedure": "Core Biopsy",
                        "type_of_primary_container": "Fecal Collection Container with NA Stabilizer",
                        "type_of_primary_container_other": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_volume": 5.0,
                        "processed_sample_volume_units": "Other",
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Sample Returned",
                    },
                    {
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TTTPP2A3')",
                        "processed_sample_id": "test_encrypted('BIOBANK 2A3')",
                        "cimac_id": "CTTTP02A3.00",
                        "box_number": "1",
                        "sample_location": "A6",
                        "type_of_sample": "Whole Blood",
                        "sample_collection_procedure": "Core Biopsy",
                        "type_of_primary_container": "Fecal Collection Container with NA Stabilizer",
                        "type_of_primary_container_other": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_volume": 6.0,
                        "processed_sample_volume_units": "Other",
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Sample Returned",
                    },
                ],
                "cohort_name": "Arm_A",
                "participant_id": "test_encrypted('TTTP02')",
                "cimac_participant_id": "CTTTP02",
            },
        ],
    }
    upload_entries: List[LocalFileUploadEntry] = []
    base_trial = get_test_trial(
        allowed_collection_event_names=["Baseline"],
        allowed_cohort_names=["Arm_A", "Arm_Z"],
    )

    target_trial = copy_dict_with_branch(
        base_trial, prismify_patch, ["participants", "shipments"]
    )

    return PrismTestData(
        upload_type,
        prismify_args,
        prismify_patch,
        upload_entries,
        base_trial,
        target_trial,
    )


@manifest_data_generator
def pbmc() -> PrismTestData:
    upload_type = "pbmc"
    prismify_args = get_prismify_args(upload_type)
    prismify_patch = {
        "protocol_identifier": "test_prism_trial_id",
        "shipments": [
            {
                "manifest_id": "test_prism_trial_id_PBMC",
                "assay_priority": "4",
                "assay_type": "Olink",
                "receiving_party": "MSSM_Rahman",
                "courier": "USPS",
                "tracking_number": "TrackN",
                "shipping_condition": "Other",
                "shipping_condition_other": "Other type of shipping condition details",
                "date_shipped": "2001-10-10 00:00:00",
                "date_received": "2002-10-10 00:00:00",
                "quality_of_shipment": "Normal",
                "ship_from_sender_person_name": "George",
                "ship_from_sender_email": "test@nih.gov",
                "ship_from_sender_phone": "111-1111",
                "ship_from_name": "Jared",
                "ship_from_street1": "abc",
                "ship_from_street2": "abc",
                "ship_from_city": "seattle",
                "ship_from_state": "WA",
                "ship_from_zip": "98003",
                "ship_from_country": "USA",
                "ship_to_name": "Marie",
                "ship_to_street1": "abc",
                "ship_to_street2": "abc",
                "ship_to_city": "Plano",
                "ship_to_state": "TX",
                "ship_to_zip": "99013",
                "ship_to_country": "USA",
            }
        ],
        "participants": [
            {
                "samples": [
                    {
                        "cimac_id": "CTTTP01A4.00",
                        "surgical_pathology_report_id": "test_encrypted('Surgical pathology report 1')",
                        "clinical_report_id": "test_encrypted('clinical report 1')",
                        "collection_event_name": "Baseline",
                        "diagnosis_verification": "Local pathology review was not consistent",
                        "site_description": "ANAL CANAL & ANUS",
                        "topography_code": "C00.1",
                        "topography_description": "LIP",
                        "histology_behavior": "8004/3",
                        "histology_behavior_description": "Neoplasm, malignant",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 1')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "box_number": "1",
                        "sample_location": "123",
                        "type_of_sample": "Whole Blood",
                        "sample_collection_procedure": "Core Biopsy",
                        "type_of_primary_container": "Fecal Collection Container with NA Stabilizer",
                        "type_of_primary_container_other": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_volume": 1.0,
                        "processed_sample_volume_units": "Other",
                        "processed_sample_concentration": 0.2,
                        "processed_sample_quantity": 3,
                        "processed_sample_concentration_units": "Not Reported",
                        "processed_sample_derivative": "ctDNA",
                        "sample_derivative_volume": 0.0,
                        "sample_derivative_volume_units": "Microliters",
                        "sample_derivative_concentration": 1.0,
                        "sample_derivative_concentration_units": "Nanogram per Microliter",
                        "pbmc_viability": 1.0,
                        "pbmc_recovery": 1.0,
                        "pbmc_resting_period_used": "Yes",
                        "material_used": 1.0,
                        "material_remaining": 0.0,
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Sample Returned",
                        "comments": "Comment",
                    },
                    {
                        "cimac_id": "CTTTP01A5.00",
                        "surgical_pathology_report_id": "test_encrypted('Surgical pathology report 2')",
                        "clinical_report_id": "test_encrypted('clinical report 2')",
                        "collection_event_name": "Pre_Day_1_Cycle_2",
                        "diagnosis_verification": "Local pathology review was not consistent",
                        "site_description": "ANAL CANAL & ANUS",
                        "topography_code": "C00.1",
                        "topography_description": "LIP",
                        "histology_behavior": "8004/3",
                        "histology_behavior_description": "Neoplasm, malignant",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 2')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "box_number": "1",
                        "sample_location": "123",
                        "type_of_sample": "Whole Blood",
                        "sample_collection_procedure": "Core Biopsy",
                        "type_of_primary_container": "Fecal Collection Container with NA Stabilizer",
                        "type_of_primary_container_other": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_volume": 2.0,
                        "processed_sample_volume_units": "Other",
                        "processed_sample_concentration": 0.3,
                        "processed_sample_concentration_units": "Not Reported",
                        "processed_sample_derivative": "ctDNA",
                        "sample_derivative_volume": 0.0,
                        "sample_derivative_volume_units": "Microliters",
                        "sample_derivative_concentration": 1.0,
                        "sample_derivative_concentration_units": "Nanogram per Microliter",
                        "pbmc_viability": 1.0,
                        "pbmc_recovery": 1.0,
                        "pbmc_resting_period_used": "No",
                        "material_used": 1.0,
                        "material_remaining": 0.0,
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Failed",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Sample Returned",
                    },
                    {
                        "cimac_id": "CTTTP01A6.00",
                        "surgical_pathology_report_id": "test_encrypted('Surgical pathology report 3')",
                        "clinical_report_id": "test_encrypted('clinical report 3')",
                        "collection_event_name": "Baseline",
                        "diagnosis_verification": "Local pathology review was not consistent",
                        "site_description": "ANAL CANAL & ANUS",
                        "topography_code": "C00.1",
                        "topography_description": "LIP",
                        "histology_behavior": "8004/3",
                        "histology_behavior_description": "Neoplasm, malignant",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 3')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "box_number": "1",
                        "sample_location": "123",
                        "type_of_sample": "Whole Blood",
                        "sample_collection_procedure": "Core Biopsy",
                        "type_of_primary_container": "Fecal Collection Container with NA Stabilizer",
                        "type_of_primary_container_other": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_volume": 3.0,
                        "processed_sample_volume_units": "Other",
                        "processed_sample_concentration": 0.2,
                        "processed_sample_concentration_units": "Not Reported",
                        "processed_sample_derivative": "ctDNA",
                        "sample_derivative_volume": 0.0,
                        "sample_derivative_volume_units": "Microliters",
                        "sample_derivative_concentration": 1.0,
                        "sample_derivative_concentration_units": "Nanogram per Microliter",
                        "pbmc_viability": 1.0,
                        "pbmc_recovery": 1.0,
                        "pbmc_resting_period_used": "Not Reported",
                        "material_used": 1.0,
                        "material_remaining": 0.0,
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Sample Returned",
                    },
                ],
                "participant_id": "test_encrypted('TTTP01')",
                "cimac_participant_id": "CTTTP01",
                "gender": "Female",
                "race": "Asian",
                "ethnicity": "Hispanic or Latino",
                "cohort_name": "Arm_Z",
            },
            {
                "samples": [
                    {
                        "cimac_id": "CTTTP02A4.00",
                        "surgical_pathology_report_id": "test_encrypted('Surgical pathology report 4')",
                        "clinical_report_id": "test_encrypted('clinical report 4')",
                        "collection_event_name": "Baseline",
                        "diagnosis_verification": "Local pathology review was not consistent",
                        "site_description": "ANAL CANAL & ANUS",
                        "topography_code": "C00.1",
                        "topography_description": "LIP",
                        "histology_behavior": "8004/3",
                        "histology_behavior_description": "Neoplasm, malignant",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 4')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "box_number": "1",
                        "sample_location": "123",
                        "type_of_sample": "Whole Blood",
                        "sample_collection_procedure": "Core Biopsy",
                        "type_of_primary_container": "Fecal Collection Container with NA Stabilizer",
                        "type_of_primary_container_other": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_volume": 4.0,
                        "processed_sample_volume_units": "Other",
                        "processed_sample_concentration": 0.2,
                        "processed_sample_concentration_units": "Not Reported",
                        "processed_sample_derivative": "ctDNA",
                        "sample_derivative_volume": 0.0,
                        "sample_derivative_volume_units": "Microliters",
                        "sample_derivative_concentration": 1.0,
                        "sample_derivative_concentration_units": "Nanogram per Microliter",
                        "pbmc_viability": 1.0,
                        "pbmc_recovery": 1.0,
                        "pbmc_resting_period_used": "Other",
                        "material_used": 1.0,
                        "material_remaining": 0.0,
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Failed",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Sample Returned",
                    },
                    {
                        "cimac_id": "CTTTP02A5.00",
                        "surgical_pathology_report_id": "test_encrypted('Surgical pathology report 5')",
                        "clinical_report_id": "test_encrypted('clinical report 5')",
                        "collection_event_name": "Pre_Day_1_Cycle_2",
                        "diagnosis_verification": "Local pathology review was not consistent",
                        "site_description": "ANAL CANAL & ANUS",
                        "topography_code": "C00.1",
                        "topography_description": "LIP",
                        "histology_behavior": "8004/3",
                        "histology_behavior_description": "Neoplasm, malignant",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 5')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "box_number": "1",
                        "sample_location": "123",
                        "type_of_sample": "Whole Blood",
                        "sample_collection_procedure": "Core Biopsy",
                        "type_of_primary_container": "Fecal Collection Container with NA Stabilizer",
                        "type_of_primary_container_other": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_volume": 5.0,
                        "processed_sample_volume_units": "Other",
                        "processed_sample_concentration": 0.2,
                        "processed_sample_concentration_units": "Not Reported",
                        "processed_sample_derivative": "ctDNA",
                        "sample_derivative_volume": 0.0,
                        "sample_derivative_volume_units": "Microliters",
                        "sample_derivative_concentration": 1.0,
                        "sample_derivative_concentration_units": "Nanogram per Microliter",
                        "pbmc_viability": 1.0,
                        "pbmc_recovery": 1.0,
                        "pbmc_resting_period_used": "Other",
                        "material_used": 1.0,
                        "material_remaining": 0.0,
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Sample Returned",
                    },
                    {
                        "cimac_id": "CTTTP02A6.00",
                        "surgical_pathology_report_id": "test_encrypted('Surgical pathology report 6')",
                        "clinical_report_id": "test_encrypted('clinical report 6')",
                        "collection_event_name": "Baseline",
                        "diagnosis_verification": "Local pathology review was not consistent",
                        "site_description": "ANAL CANAL & ANUS",
                        "topography_code": "C00.1",
                        "topography_description": "LIP",
                        "histology_behavior": "8004/3",
                        "histology_behavior_description": "Neoplasm, malignant",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 6')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "box_number": "1",
                        "sample_location": "123",
                        "type_of_sample": "Whole Blood",
                        "sample_collection_procedure": "Core Biopsy",
                        "type_of_primary_container": "Fecal Collection Container with NA Stabilizer",
                        "type_of_primary_container_other": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_volume": 6.0,
                        "processed_sample_volume_units": "Other",
                        "processed_sample_concentration": 0.3,
                        "processed_sample_concentration_units": "Not Reported",
                        "processed_sample_derivative": "ctDNA",
                        "sample_derivative_volume": 0.0,
                        "sample_derivative_volume_units": "Microliters",
                        "sample_derivative_concentration": 1.0,
                        "sample_derivative_concentration_units": "Nanogram per Microliter",
                        "pbmc_viability": 1.0,
                        "pbmc_recovery": 1.0,
                        "pbmc_resting_period_used": "Other",
                        "material_used": 1.0,
                        "material_remaining": 0.0,
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Failed",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Sample Returned",
                    },
                ],
                "participant_id": "test_encrypted('TTTP02')",
                "cimac_participant_id": "CTTTP02",
                "gender": "Male",
                "race": "Native Hawaiian/Pacific Islander",
                "ethnicity": "Unknown",
                "cohort_name": "Arm_A",
            },
        ],
    }
    upload_entries = []
    base_trial = get_test_trial(
        allowed_cohort_names=["Arm_A", "Arm_Z"],
        allowed_collection_event_names=["Baseline", "Pre_Day_1_Cycle_2"],
    )

    target_trial = copy_dict_with_branch(
        base_trial, prismify_patch, ["participants", "shipments"]
    )

    return PrismTestData(
        upload_type,
        prismify_args,
        prismify_patch,
        upload_entries,
        base_trial,
        target_trial,
    )


@manifest_data_generator
def tissue_slide() -> PrismTestData:
    upload_type = "tissue_slide"
    prismify_args = get_prismify_args(upload_type)
    prismify_patch = {
        "protocol_identifier": "test_prism_trial_id",
        "shipments": [
            {
                "manifest_id": "test_prism_trial_id_slide",
                "assay_priority": "3",
                "assay_type": "IHC",
                "receiving_party": "DFCI_Severgnini",
                "courier": "DHL",
                "tracking_number": "TrackN",
                "shipping_condition": "Not Reported",
                "date_shipped": "2001-10-10 00:00:00",
                "date_received": "2002-10-10 00:00:00",
                "quality_of_shipment": "Normal",
                "ship_from_sender_person_name": "George",
                "ship_from_sender_email": "test@nih.gov",
                "ship_from_sender_phone": "111-1111",
                "ship_from_name": "Jared",
                "ship_from_street1": "abc",
                "ship_from_street2": "abc",
                "ship_from_city": "seattle",
                "ship_from_state": "WA",
                "ship_from_zip": "98003",
                "ship_from_country": "USA",
                "ship_to_name": "Marie",
                "ship_to_street1": "abc",
                "ship_to_street2": "abc",
                "ship_to_city": "Plano",
                "ship_to_state": "TX",
                "ship_to_zip": "99013",
                "ship_to_country": "USA",
            }
        ],
        "participants": [
            {
                "samples": [
                    {
                        "shipping_entry_number": 1,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 1')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP08T1.00",
                        "box_number": "2",
                        "sample_location": "A1",
                        "type_of_sample": "Fresh Tissue",
                        "type_of_tumor_sample": "Metastatic Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 4.0,
                        "material_used": 3.0,
                        "material_used_units": "Slides",
                        "material_remaining": 1.0,
                        "material_remaining_units": "Slides",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                    {
                        "shipping_entry_number": 2,
                        "collection_event_name": "Pre_Day_1_Cycle_2",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 2')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP08T2.00",
                        "box_number": "2",
                        "sample_location": "A2",
                        "type_of_sample": "Fresh Tissue",
                        "type_of_tumor_sample": "Metastatic Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "H&E",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 4.0,
                        "material_used": 3.0,
                        "material_used_units": "Slides",
                        "material_remaining": 1.0,
                        "material_remaining_units": "Slides",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                    {
                        "shipping_entry_number": 3,
                        "collection_event_name": "Pre_Day_1_Cycle_2",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 3')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP08T3.00",
                        "box_number": "2",
                        "sample_location": "A3",
                        "type_of_sample": "Fresh Tissue",
                        "type_of_tumor_sample": "Primary Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Proteomic Stabilization",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "material_used": 1.0,
                        "material_used_units": "Slides",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Slides",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                ],
                "cohort_name": "Arm_A",
                "participant_id": "test_encrypted('TTTPP8')",
                "cimac_participant_id": "CTTTP08",
            },
            {
                "samples": [
                    {
                        "shipping_entry_number": 4,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 4')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP09T1.00",
                        "box_number": "2",
                        "sample_location": "A4",
                        "type_of_sample": "Fresh Tissue",
                        "type_of_tumor_sample": "Primary Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Not Reported",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "material_used": 1.0,
                        "material_used_units": "Slides",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Slides",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    }
                ],
                "cohort_name": "Arm_Z",
                "participant_id": "test_encrypted('TTTPP901')",
                "cimac_participant_id": "CTTTP09",
            },
        ],
    }
    upload_entries = []
    base_trial = get_test_trial(
        allowed_collection_event_names=["Baseline", "Pre_Day_1_Cycle_2"],
    )

    target_trial = copy_dict_with_branch(
        base_trial, prismify_patch, ["participants", "shipments"]
    )

    return PrismTestData(
        upload_type,
        prismify_args,
        prismify_patch,
        upload_entries,
        base_trial,
        target_trial,
    )


@manifest_data_generator
def normal_blood_dna() -> PrismTestData:
    upload_type = "normal_blood_dna"
    prismify_args = get_prismify_args(upload_type)
    prismify_patch = {
        "protocol_identifier": "test_prism_trial_id",
        "shipments": [
            {
                "manifest_id": "test_prism_trial_id_DNA",
                "assay_priority": "4",
                "assay_type": "WES",
                "receiving_party": "MDA_Bernatchez",
                "courier": "USPS",
                "tracking_number": "TrackN",
                "shipping_condition": "Ice/Cold Pack",
                "date_shipped": "2001-10-10 00:00:00",
                "date_received": "2002-10-10 00:00:00",
                "quality_of_shipment": "Normal",
                "ship_from_sender_person_name": "George",
                "ship_from_sender_email": "test@nih.gov",
                "ship_from_sender_phone": "111-1111",
                "ship_from_name": "Jared",
                "ship_from_street1": "abc",
                "ship_from_street2": "abc",
                "ship_from_city": "seattle",
                "ship_from_state": "WA",
                "ship_from_zip": "98003",
                "ship_from_country": "USA",
                "ship_to_name": "Marie",
                "ship_to_street1": "abc",
                "ship_to_street2": "abc",
                "ship_to_city": "Plano",
                "ship_to_state": "TX",
                "ship_to_zip": "99013",
                "ship_to_country": "USA",
            }
        ],
        "participants": [
            {
                "samples": [
                    {
                        "shipping_entry_number": 1,
                        "collection_event_name": "Baseline",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 1')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP01N1.00",
                        "box_number": "1",
                        "sample_location": "A1",
                        "type_of_sample": "Whole Blood",
                        "sample_collection_procedure": "Surgical Excision",
                        "type_of_primary_container": "EDTA Tube",
                        "type_of_primary_container_other": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_volume": 1.0,
                        "processed_sample_volume_units": "Other",
                        "processed_sample_quantity": 1.0,
                        "processed_sample_derivative": "ctDNA",
                        "sample_derivative_volume": 1.0,
                        "sample_derivative_volume_units": "Microliters",
                        "sample_derivative_concentration": 1.0,
                        "sample_derivative_concentration_units": "Nanogram per Microliter",
                        "din": 9.0,
                        "a260_a280": 1.8,
                        "a260_a230": 2.0,
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                        "comments": "Comment",
                    },
                    {
                        "shipping_entry_number": 2,
                        "collection_event_name": "Baseline",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 2')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP01N2.00",
                        "box_number": "1",
                        "sample_location": "A2",
                        "type_of_sample": "Whole Blood",
                        "type_of_sample_other": "other_sample",
                        "sample_collection_procedure": "Surgical Excision",
                        "type_of_primary_container": "EDTA Tube",
                        "type_of_primary_container_other": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_volume": 2.0,
                        "processed_sample_volume_units": "Other",
                        "processed_sample_quantity": 2.0,
                        "processed_sample_derivative": "Germline RNA",
                        "sample_derivative_volume": 2.0,
                        "sample_derivative_volume_units": "Milliliters",
                        "sample_derivative_concentration": 2.0,
                        "sample_derivative_concentration_units": "Milligram per Milliliter",
                        "din": 9.0,
                        "a260_a280": 1.7,
                        "a260_a230": 2.1,
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                        "comments": "No comment",
                    },
                    {
                        "shipping_entry_number": 3,
                        "collection_event_name": "Baseline",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 3')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP01N3.00",
                        "box_number": "1",
                        "sample_location": "A3",
                        "type_of_sample": "Whole Blood",
                        "sample_collection_procedure": "Surgical Excision",
                        "type_of_primary_container": "EDTA Tube",
                        "type_of_primary_container_other": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_volume": 3.0,
                        "processed_sample_volume_units": "Other",
                        "processed_sample_quantity": 3.0,
                        "processed_sample_derivative": "Germline DNA",
                        "sample_derivative_volume": 3.0,
                        "sample_derivative_volume_units": "Microliters",
                        "sample_derivative_concentration": 3.0,
                        "sample_derivative_concentration_units": "Micrograms per Microliter",
                        "din": 8.0,
                        "a260_a280": 1.9,
                        "a260_a230": 2.2,
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                ],
                "cohort_name": "Arm_A",
                "participant_id": "test_encrypted('TTTPP1')",
                "cimac_participant_id": "CTTTP01",
            },
            {
                "samples": [
                    {
                        "shipping_entry_number": 4,
                        "collection_event_name": "Baseline",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 4')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP02N1.00",
                        "box_number": "1",
                        "sample_location": "A4",
                        "type_of_sample": "Whole Blood",
                        "sample_collection_procedure": "Surgical Excision",
                        "type_of_primary_container": "EDTA Tube",
                        "type_of_primary_container_other": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_volume": 4.0,
                        "processed_sample_volume_units": "Other",
                        "processed_sample_quantity": 4.0,
                        "processed_sample_derivative": "ctDNA",
                        "sample_derivative_volume": 4.0,
                        "sample_derivative_volume_units": "Milliliters",
                        "sample_derivative_concentration": 4.0,
                        "sample_derivative_concentration_units": "Cells per Vial",
                        "din": 9.0,
                        "a260_a280": 1.8,
                        "a260_a230": 2.0,
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                    {
                        "shipping_entry_number": 5,
                        "collection_event_name": "Baseline",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 5')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP02N2.00",
                        "box_number": "1",
                        "sample_location": "A5",
                        "type_of_sample": "Whole Blood",
                        "sample_collection_procedure": "Surgical Excision",
                        "type_of_primary_container": "EDTA Tube",
                        "type_of_primary_container_other": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_volume": 5.0,
                        "processed_sample_volume_units": "Other",
                        "processed_sample_quantity": 5.0,
                        "processed_sample_derivative": "Other",
                        "sample_derivative_volume": 5.0,
                        "sample_derivative_volume_units": "Not Reported",
                        "sample_derivative_concentration": 5.0,
                        "sample_derivative_concentration_units": "Not Reported",
                        "din": 9.0,
                        "a260_a280": 1.7,
                        "a260_a230": 2.1,
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                    {
                        "shipping_entry_number": 6,
                        "collection_event_name": "Baseline",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 6')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP02N3.00",
                        "box_number": "1",
                        "sample_location": "A6",
                        "type_of_sample": "Whole Blood",
                        "sample_collection_procedure": "Surgical Excision",
                        "type_of_primary_container": "EDTA Tube",
                        "type_of_primary_container_other": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_volume": 6.0,
                        "processed_sample_volume_units": "Other",
                        "processed_sample_quantity": 6.0,
                        "processed_sample_derivative": "Other",
                        "sample_derivative_volume": 6.0,
                        "sample_derivative_volume_units": "Other",
                        "sample_derivative_concentration": 6.0,
                        "sample_derivative_concentration_units": "Other",
                        "din": 8.0,
                        "a260_a280": 1.9,
                        "a260_a230": 2.2,
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                ],
                "cohort_name": "Arm_Z",
                "participant_id": "test_encrypted('TTTPP2')",
                "cimac_participant_id": "CTTTP02",
            },
        ],
    }
    upload_entries = []
    base_trial = get_test_trial(
        allowed_cohort_names=["Arm_A", "Arm_Z"],
    )

    target_trial = copy_dict_with_branch(
        base_trial, prismify_patch, ["participants", "shipments"]
    )

    return PrismTestData(
        upload_type,
        prismify_args,
        prismify_patch,
        upload_entries,
        base_trial,
        target_trial,
    )


@manifest_data_generator
def normal_tissue_dna() -> PrismTestData:
    upload_type = "normal_tissue_dna"
    prismify_args = get_prismify_args(upload_type)
    prismify_patch = {
        "protocol_identifier": "test_prism_trial_id",
        "shipments": [
            {
                "manifest_id": "test_prism_trial_id_DNA",
                "assay_priority": "4",
                "assay_type": "WES",
                "receiving_party": "Broad_Cibulskis",
                "courier": "FedEx",
                "tracking_number": "TrackN",
                "shipping_condition": "Other",
                "shipping_condition_other": "Other type of shipping condition details",
                "date_shipped": "2001-10-10 00:00:00",
                "date_received": "2002-10-10 00:00:00",
                "quality_of_shipment": "Normal",
                "ship_from_sender_person_name": "George",
                "ship_from_sender_email": "test@nih.gov",
                "ship_from_sender_phone": "111-1111",
                "ship_from_name": "Jared",
                "ship_from_street1": "abc",
                "ship_from_street2": "abc",
                "ship_from_city": "seattle",
                "ship_from_state": "WA",
                "ship_from_zip": "98003",
                "ship_from_country": "USA",
                "ship_to_name": "Marie",
                "ship_to_street1": "abc",
                "ship_to_street2": "abc",
                "ship_to_city": "Plano",
                "ship_to_state": "TX",
                "ship_to_zip": "99013",
                "ship_to_country": "USA",
            }
        ],
        "participants": [
            {
                "samples": [
                    {
                        "shipping_entry_number": 1,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 1')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP03N1.00",
                        "box_number": "1",
                        "sample_location": "A1",
                        "type_of_sample": "Skin Tissue",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "H&E",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "processed_sample_derivative": "Germline DNA",
                        "sample_derivative_volume": 1.0,
                        "sample_derivative_volume_units": "Microliters",
                        "sample_derivative_concentration": 1.0,
                        "sample_derivative_concentration_units": "Nanogram per Microliter",
                        "din": 9.0,
                        "a260_a280": 1.8,
                        "a260_a230": 2.0,
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                        "comments": "Comment",
                    },
                    {
                        "shipping_entry_number": 2,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 2')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP03N2.00",
                        "box_number": "1",
                        "sample_location": "A2",
                        "type_of_sample": "Skin Tissue",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "processed_sample_derivative": "Germline DNA",
                        "sample_derivative_volume": 2.0,
                        "sample_derivative_volume_units": "Milliliters",
                        "sample_derivative_concentration": 2.0,
                        "sample_derivative_concentration_units": "Milligram per Milliliter",
                        "din": 9.0,
                        "a260_a280": 1.7,
                        "a260_a230": 2.1,
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                        "comments": "No comment",
                    },
                    {
                        "shipping_entry_number": 3,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 3')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP03N3.00",
                        "box_number": "1",
                        "sample_location": "A3",
                        "type_of_sample": "Skin Tissue",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 2.0,
                        "processed_sample_derivative": "Germline DNA",
                        "sample_derivative_volume": 3.0,
                        "sample_derivative_volume_units": "Microliters",
                        "sample_derivative_concentration": 3.0,
                        "sample_derivative_concentration_units": "Micrograms per Microliter",
                        "din": 8.0,
                        "a260_a280": 1.9,
                        "a260_a230": 2.2,
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                ],
                "cohort_name": "Arm_A",
                "participant_id": "test_encrypted('TTTPP3')",
                "cimac_participant_id": "CTTTP03",
            },
            {
                "samples": [
                    {
                        "shipping_entry_number": 4,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 4')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP04N1.00",
                        "box_number": "1",
                        "sample_location": "A4",
                        "type_of_sample": "Skin Tissue",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Proteomic Stabilization",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "processed_sample_derivative": "Germline DNA",
                        "sample_derivative_volume": 4.0,
                        "sample_derivative_volume_units": "Milliliters",
                        "sample_derivative_concentration": 4.0,
                        "sample_derivative_concentration_units": "Cells per Vial",
                        "din": 9.0,
                        "a260_a280": 1.8,
                        "a260_a230": 2.0,
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                    {
                        "shipping_entry_number": 5,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 5')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP04N2.00",
                        "box_number": "1",
                        "sample_location": "A5",
                        "type_of_sample": "Skin Tissue",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Liquid Nitrogen (Frozen)",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "processed_sample_derivative": "Germline DNA",
                        "sample_derivative_volume": 5.0,
                        "sample_derivative_volume_units": "Not Reported",
                        "sample_derivative_concentration": 5.0,
                        "sample_derivative_concentration_units": "Not Reported",
                        "din": 9.0,
                        "a260_a280": 1.7,
                        "a260_a230": 2.1,
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                    {
                        "shipping_entry_number": 6,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 6')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP04N3.00",
                        "box_number": "1",
                        "sample_location": "A6",
                        "type_of_sample": "Skin Tissue",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Thaw-Lyse",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 2.0,
                        "processed_sample_derivative": "Germline DNA",
                        "sample_derivative_volume": 6.0,
                        "sample_derivative_volume_units": "Other",
                        "sample_derivative_concentration": 6.0,
                        "sample_derivative_concentration_units": "Other",
                        "din": 8.0,
                        "a260_a280": 1.9,
                        "a260_a230": 2.2,
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                ],
                "cohort_name": "Arm_Z",
                "participant_id": "test_encrypted('TTTPP4')",
                "cimac_participant_id": "CTTTP04",
            },
        ],
    }
    upload_entries = []
    base_trial = get_test_trial(
        allowed_cohort_names=["Arm_A", "Arm_Z"],
        allowed_collection_event_names=["Baseline", "Pre_Day_1_Cycle_2"],
    )

    target_trial = copy_dict_with_branch(
        base_trial, prismify_patch, ["participants", "shipments"]
    )

    return PrismTestData(
        upload_type,
        prismify_args,
        prismify_patch,
        upload_entries,
        base_trial,
        target_trial,
    )


@manifest_data_generator
def tumor_tissue_dna() -> PrismTestData:
    upload_type = "tumor_tissue_dna"
    prismify_args = get_prismify_args(upload_type)
    prismify_patch = {
        "protocol_identifier": "test_prism_trial_id",
        "shipments": [
            {
                "manifest_id": "test_prism_trial_id_DNA",
                "assay_priority": "4",
                "assay_type": "WES",
                "receiving_party": "MDA_Wistuba",
                "courier": "FedEx",
                "tracking_number": "TrackN",
                "shipping_condition": "Ice/Cold Pack",
                "date_shipped": "2001-10-10 00:00:00",
                "date_received": "2002-10-10 00:00:00",
                "quality_of_shipment": "Normal",
                "ship_from_sender_person_name": "George",
                "ship_from_sender_email": "test@nih.gov",
                "ship_from_sender_phone": "111-1111",
                "ship_from_name": "Jared",
                "ship_from_street1": "abc",
                "ship_from_street2": "abc",
                "ship_from_city": "seattle",
                "ship_from_state": "WA",
                "ship_from_zip": "98003",
                "ship_from_country": "USA",
                "ship_to_name": "Marie",
                "ship_to_street1": "abc",
                "ship_to_street2": "abc",
                "ship_to_city": "Plano",
                "ship_to_state": "TX",
                "ship_to_zip": "99013",
                "ship_to_country": "USA",
            }
        ],
        "participants": [
            {
                "samples": [
                    {
                        "shipping_entry_number": 1,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 1')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP05T1.00",
                        "box_number": "1",
                        "sample_location": "A1",
                        "type_of_sample": "Skin Tissue",
                        "type_of_tumor_sample": "Metastatic Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "processed_sample_derivative": "Germline DNA",
                        "sample_derivative_volume": 1.0,
                        "sample_derivative_volume_units": "Microliters",
                        "sample_derivative_concentration": 1.0,
                        "sample_derivative_concentration_units": "Nanogram per Microliter",
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                        "comments": "Comment",
                    },
                    {
                        "shipping_entry_number": 2,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 2')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP05T2.00",
                        "box_number": "1",
                        "sample_location": "A2",
                        "type_of_sample": "Skin Tissue",
                        "type_of_tumor_sample": "Metastatic Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "processed_sample_derivative": "Germline DNA",
                        "sample_derivative_volume": 2.0,
                        "sample_derivative_volume_units": "Milliliters",
                        "sample_derivative_concentration": 2.0,
                        "sample_derivative_concentration_units": "Milligram per Milliliter",
                        "din": 9.0,
                        "a260_a280": 1.7,
                        "a260_a230": 2.1,
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                        "comments": "No comment",
                    },
                    {
                        "shipping_entry_number": 3,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 3')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP05T3.00",
                        "box_number": "1",
                        "sample_location": "A3",
                        "type_of_sample": "Skin Tissue",
                        "type_of_tumor_sample": "Primary Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 2.0,
                        "processed_sample_derivative": "Germline DNA",
                        "sample_derivative_volume": 3.0,
                        "sample_derivative_volume_units": "Microliters",
                        "sample_derivative_concentration": 3.0,
                        "sample_derivative_concentration_units": "Micrograms per Microliter",
                        "a260_a280": 1.9,
                        "a260_a230": 2.2,
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                ],
                "participant_id": "test_encrypted('TTTPP5')",
                "cimac_participant_id": "CTTTP05",
            },
            {
                "samples": [
                    {
                        "shipping_entry_number": 4,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 4')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP06T1.00",
                        "box_number": "1",
                        "sample_location": "A4",
                        "type_of_sample": "Skin Tissue",
                        "type_of_tumor_sample": "Primary Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Proteomic Stabilization",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "processed_sample_derivative": "Germline DNA",
                        "sample_derivative_volume": 4.0,
                        "sample_derivative_volume_units": "Milliliters",
                        "sample_derivative_concentration": 4.0,
                        "sample_derivative_concentration_units": "Cells per Vial",
                        "din": 9.0,
                        "a260_a230": 2.0,
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                    {
                        "shipping_entry_number": 5,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 5')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP06T2.00",
                        "box_number": "1",
                        "sample_location": "A5",
                        "type_of_sample": "Skin Tissue",
                        "type_of_tumor_sample": "Primary Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Liquid Nitrogen (Frozen)",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "processed_sample_derivative": "Germline DNA",
                        "sample_derivative_volume": 5.0,
                        "sample_derivative_volume_units": "Not Reported",
                        "sample_derivative_concentration": 5.0,
                        "sample_derivative_concentration_units": "Not Reported",
                        "din": 9.0,
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                    {
                        "shipping_entry_number": 6,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 6')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP06T3.00",
                        "box_number": "1",
                        "sample_location": "A6",
                        "type_of_sample": "Skin Tissue",
                        "type_of_tumor_sample": "Primary Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Thaw-Lyse",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 2.0,
                        "processed_sample_derivative": "Germline DNA",
                        "sample_derivative_volume": 6.0,
                        "sample_derivative_volume_units": "Other",
                        "sample_derivative_concentration": 6.0,
                        "sample_derivative_concentration_units": "Other",
                        "a260_a230": 2.2,
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                ],
                "cohort_name": "Arm_Z",
                "participant_id": "test_encrypted('TTTPP6')",
                "cimac_participant_id": "CTTTP06",
            },
        ],
    }
    upload_entries = []
    base_trial = get_test_trial(
        allowed_cohort_names=["Arm_A", "Arm_Z"],
        allowed_collection_event_names=["Baseline", "Pre_Day_1_Cycle_2"],
    )

    target_trial = copy_dict_with_branch(
        base_trial, prismify_patch, ["participants", "shipments"]
    )

    return PrismTestData(
        upload_type,
        prismify_args,
        prismify_patch,
        upload_entries,
        base_trial,
        target_trial,
    )


@manifest_data_generator
def h_and_e() -> PrismTestData:
    upload_type = "h_and_e"
    prismify_args = get_prismify_args(upload_type)
    prismify_patch = {
        "protocol_identifier": "test_prism_trial_id",
        "shipments": [
            {
                "manifest_id": "test_prism_trial_id_slide",
                "assay_priority": "3",
                "assay_type": "IHC",
                "receiving_party": "Broad_Cibulskis",
                "courier": "USPS",
                "tracking_number": "TrackN",
                "shipping_condition": "Not Reported",
                "date_shipped": "2001-10-10 00:00:00",
                "date_received": "2002-10-10 00:00:00",
                "quality_of_shipment": "Normal",
                "ship_from_sender_person_name": "George",
                "ship_from_sender_email": "test@nih.gov",
                "ship_from_sender_phone": "111-1111",
                "ship_from_name": "Jared",
                "ship_from_street1": "abc",
                "ship_from_street2": "abc",
                "ship_from_city": "seattle",
                "ship_from_state": "WA",
                "ship_from_zip": "98003",
                "ship_from_country": "USA",
                "ship_to_name": "Marie",
                "ship_to_street1": "abc",
                "ship_to_street2": "abc",
                "ship_to_city": "Plano",
                "ship_to_state": "TX",
                "ship_to_zip": "99013",
                "ship_to_country": "USA",
            }
        ],
        "participants": [
            {
                "samples": [
                    {
                        "shipping_entry_number": 1,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 1')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP08T1.00",
                        "box_number": "2",
                        "sample_location": "A1",
                        "type_of_sample": "Fresh Tissue",
                        "type_of_tumor_sample": "Primary Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Proteomic Stabilization",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "tumor_tissue_total_area_percentage": 0.0,
                        "viable_tumor_area_percentage": 0.0,
                        "viable_stroma_area_percentage": 0.0,
                        "necrosis_area_percentage": 0.0,
                        "fibrosis_area_percentage": 0.0,
                        "material_used": 1.0,
                        "material_used_units": "Slides",
                        "material_remaining": 1.0,
                        "material_remaining_units": "Slides",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                    {
                        "shipping_entry_number": 2,
                        "collection_event_name": "Pre_Day_1_Cycle_2",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 2')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP08T2.00",
                        "box_number": "2",
                        "sample_location": "A2",
                        "type_of_sample": "Fresh Tissue",
                        "type_of_tumor_sample": "Primary Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Proteomic Stabilization",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "tumor_tissue_total_area_percentage": 100.0,
                        "viable_tumor_area_percentage": 100.0,
                        "viable_stroma_area_percentage": 100.0,
                        "necrosis_area_percentage": 100.0,
                        "fibrosis_area_percentage": 100.0,
                        "material_used": 1.0,
                        "material_used_units": "Slides",
                        "material_remaining": 1.0,
                        "material_remaining_units": "Slides",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                    {
                        "shipping_entry_number": 3,
                        "collection_event_name": "Pre_Day_1_Cycle_2",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 2')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP08T3.01",
                        "box_number": "2",
                        "sample_location": "A2",
                        "type_of_sample": "Fresh Tissue",
                        "type_of_tumor_sample": "Primary Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Proteomic Stabilization",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "tumor_tissue_total_area_percentage": "Not Reported",
                        "viable_tumor_area_percentage": 100.0,
                        "viable_stroma_area_percentage": 100.0,
                        "necrosis_area_percentage": 100.0,
                        "fibrosis_area_percentage": 100.0,
                        "material_used": 1.0,
                        "material_used_units": "Slides",
                        "material_remaining": 1.0,
                        "material_remaining_units": "Slides",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                ],
                "cohort_name": "Arm_A",
                "participant_id": "test_encrypted('TTTPP8')",
                "cimac_participant_id": "CTTTP08",
            }
        ],
    }
    upload_entries = []
    base_trial = get_test_trial(
        allowed_cohort_names=["Arm_A", "Arm_Z"],
        allowed_collection_event_names=["Baseline", "Pre_Day_1_Cycle_2"],
    )

    target_trial = copy_dict_with_branch(
        base_trial, prismify_patch, ["participants", "shipments"]
    )

    return PrismTestData(
        upload_type,
        prismify_args,
        prismify_patch,
        upload_entries,
        base_trial,
        target_trial,
    )


@manifest_data_generator
def tumor_tissue_rna() -> PrismTestData:
    upload_type = "tumor_tissue_rna"
    prismify_args = get_prismify_args(upload_type)
    prismify_patch = {
        "protocol_identifier": "test_prism_trial_id",
        "shipments": [
            {
                "manifest_id": "test_prism_trial_tumor_tissue_RNA",
                "assay_priority": "4",
                "assay_type": "RNAseq",
                "receiving_party": "MDA_Wistuba",
                "courier": "USPS",
                "tracking_number": "TrackN",
                "shipping_condition": "Ice/Cold Pack",
                "date_shipped": "2001-10-10 00:00:00",
                "date_received": "2002-10-10 00:00:00",
                "quality_of_shipment": "Normal",
                "ship_from_sender_person_name": "George",
                "ship_from_sender_email": "test@nih.gov",
                "ship_from_sender_phone": "111-1111",
                "ship_from_name": "Jared",
                "ship_from_street1": "abc",
                "ship_from_street2": "abc",
                "ship_from_city": "seattle",
                "ship_from_state": "WA",
                "ship_from_zip": "98003",
                "ship_from_country": "USA",
                "ship_to_name": "Marie",
                "ship_to_street1": "abc",
                "ship_to_street2": "abc",
                "ship_to_city": "Plano",
                "ship_to_state": "TX",
                "ship_to_zip": "99013",
                "ship_to_country": "USA",
            }
        ],
        "participants": [
            {
                "samples": [
                    {
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 1')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP05T1.00",
                        "box_number": "1",
                        "sample_location": "A1",
                        "type_of_sample": "Skin Tissue",
                        "type_of_tumor_sample": "Metastatic Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "processed_sample_derivative": "Germline RNA",
                        "sample_derivative_volume": 1.0,
                        "sample_derivative_volume_units": "Microliters",
                        "sample_derivative_concentration": 1.0,
                        "sample_derivative_concentration_units": "Nanogram per Microliter",
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                        "comments": "Comment",
                    },
                    {
                        "shipping_entry_number": 2,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 2')",
                        "processed_sample_id": "test_encrypted('BIOBANK 2')",
                        "cimac_id": "CTTTP05T2.00",
                        "box_number": "1",
                        "sample_location": "A2",
                        "type_of_sample": "Skin Tissue",
                        "type_of_tumor_sample": "Metastatic Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "processed_sample_derivative": "Germline RNA",
                        "sample_derivative_volume": 2.0,
                        "sample_derivative_volume_units": "Milliliters",
                        "sample_derivative_concentration": 2.0,
                        "sample_derivative_concentration_units": "Milligram per Milliliter",
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                        "comments": "No comment",
                    },
                    {
                        "shipping_entry_number": 3,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 3')",
                        "processed_sample_id": "test_encrypted('BIOBANK 3')",
                        "cimac_id": "CTTTP05T3.00",
                        "box_number": "1",
                        "sample_location": "A3",
                        "type_of_sample": "Skin Tissue",
                        "type_of_tumor_sample": "Primary Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 2.0,
                        "processed_sample_derivative": "Germline RNA",
                        "sample_derivative_volume": 3.0,
                        "sample_derivative_volume_units": "Microliters",
                        "sample_derivative_concentration": 3.0,
                        "sample_derivative_concentration_units": "Micrograms per Microliter",
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                ],
                "cohort_name": "Arm_A",
                "participant_id": "test_encrypted('TTTPP5')",
                "cimac_participant_id": "CTTTP05",
            },
            {
                "samples": [
                    {
                        "shipping_entry_number": 4,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 4')",
                        "processed_sample_id": "test_encrypted('BIOBANK 4')",
                        "cimac_id": "CTTTP06T1.00",
                        "box_number": "1",
                        "sample_location": "A4",
                        "type_of_sample": "Skin Tissue",
                        "type_of_tumor_sample": "Primary Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "processed_sample_derivative": "Germline RNA",
                        "sample_derivative_volume": 4.0,
                        "sample_derivative_volume_units": "Milliliters",
                        "sample_derivative_concentration": 4.0,
                        "sample_derivative_concentration_units": "Cells per Vial",
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                    {
                        "shipping_entry_number": 5,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 5')",
                        "processed_sample_id": "test_encrypted('BIOBANK 5')",
                        "cimac_id": "CTTTP06T2.00",
                        "box_number": "1",
                        "sample_location": "A5",
                        "type_of_sample": "Skin Tissue",
                        "type_of_tumor_sample": "Primary Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Liquid Nitrogen (Frozen)",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "processed_sample_derivative": "Germline RNA",
                        "sample_derivative_volume": 5.0,
                        "sample_derivative_volume_units": "Not Reported",
                        "sample_derivative_concentration": 5.0,
                        "sample_derivative_concentration_units": "Not Reported",
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                    {
                        "shipping_entry_number": 6,
                        "collection_event_name": "Baseline",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 6')",
                        "processed_sample_id": "test_encrypted('BIOBANK 6')",
                        "cimac_id": "CTTTP06T3.00",
                        "box_number": "1",
                        "sample_location": "A6",
                        "type_of_sample": "Skin Tissue",
                        "type_of_tumor_sample": "Primary Tumor",
                        "description_of_sample": "Normal",
                        "sample_collection_procedure": "Core Biopsy",
                        "core_number": "1.0",
                        "fixation_stabilization_type": "Thaw-Lyse",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 2.0,
                        "processed_sample_derivative": "Germline RNA",
                        "sample_derivative_volume": 6.0,
                        "sample_derivative_volume_units": "Other",
                        "sample_derivative_concentration": 6.0,
                        "sample_derivative_concentration_units": "Other",
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                ],
                "participant_id": "test_encrypted('TTTPP6')",
                "cimac_participant_id": "CTTTP06",
            },
        ],
    }
    upload_entries = []
    base_trial = get_test_trial()

    target_trial = copy_dict_with_branch(
        base_trial, prismify_patch, ["participants", "shipments"]
    )

    return PrismTestData(
        upload_type,
        prismify_args,
        prismify_patch,
        upload_entries,
        base_trial,
        target_trial,
    )


@manifest_data_generator
def tumor_normal_pairing():
    upload_type = "tumor_normal_pairing"
    prismify_args = get_prismify_args(upload_type)
    prismify_patch = {
        "protocol_identifier": "test_prism_trial_id",
        "analysis": {
            "wes_analysis": {
                "pair_runs": [
                    {
                        "tumor": {"cimac_id": "CTTTPP111.00"},
                        "run_id": "CTTTPP111.00",
                        "normal": {"cimac_id": "CTTTPP121.00"},
                    }
                ]
            },
            "wes_tumor_only_analysis": {
                "runs": [
                    {
                        "run_id": cimac_id,
                        "tumor": {"cimac_id": cimac_id},
                    }
                    for cimac_id in [
                        "CTTTPP122.00",
                        "CTTTPP123.00",
                        "CTTTPP124.00",
                        "CTTTPP211.00",
                        "CTTTPP212.00",
                        "CTTTPP213.00",
                        "CTTTPP214.00",
                        "CTTTPP311.00",
                        "CTTTPP312.00",
                        "CTTTPP313.00",
                        "CTTTPP411.00",
                        "CTTTPP412.00",
                        "CTTTPP413.00",
                        "CTTTPP511.00",
                        "CTTTPP512.00",
                        "CTTTPP513.00",
                        "CTTTPP514.00",
                        "CTTTPP515.00",
                        "CTTTPP516.00",
                        "CTTTPP517.00",
                    ]
                ],
            },
        },
    }
    upload_entries = []

    cimac_ids = [
        sample["cimac_id"]
        for pair_run in prismify_patch["analysis"]["wes_analysis"]["pair_runs"]
        for sample in [pair_run["tumor"], pair_run["normal"]]
    ] + [
        run["tumor"]["cimac_id"]
        for run in prismify_patch["analysis"]["wes_tumor_only_analysis"]["runs"]
    ]
    base_trial = get_test_trial(cimac_ids)

    target_trial = copy_dict_with_branch(base_trial, prismify_patch, "analysis")

    return PrismTestData(
        upload_type,
        prismify_args,
        prismify_patch,
        upload_entries,
        base_trial,
        target_trial,
    )


@manifest_data_generator
def microbiome_dna() -> PrismTestData:
    upload_type = "microbiome_dna"
    prismify_args = get_prismify_args(upload_type)
    prismify_patch = {
        "protocol_identifier": "test_prism_trial_id",
        "shipments": [
            {
                "manifest_id": "test_prism_trial_id",
                "assay_priority": "3",
                "assay_type": "Microbiome",
                "receiving_party": "DFCI_Severgnini",
                "courier": "DHL",
                "tracking_number": "TrackN",
                "shipping_condition": "Other",
                "shipping_condition_other": "Other type of shipping condition details",
                "date_shipped": "2001-10-10 00:00:00",
                "date_received": "2002-10-10 00:00:00",
                "quality_of_shipment": "Normal",
                "ship_from_sender_person_name": "George",
                "ship_from_sender_email": "test@nih.gov",
                "ship_from_sender_phone": "111-1111",
                "ship_from_name": "Jared",
                "ship_from_street1": "abc",
                "ship_from_street2": "abc",
                "ship_from_city": "seattle",
                "ship_from_state": "WA",
                "ship_from_zip": "98003",
                "ship_from_country": "USA",
                "ship_to_name": "Marie",
                "ship_to_street1": "abc",
                "ship_to_street2": "abc",
                "ship_to_city": "Plano",
                "ship_to_state": "TX",
                "ship_to_zip": "99013",
                "ship_to_country": "USA",
            }
        ],
        "participants": [
            {
                "samples": [
                    {
                        "shipping_entry_number": 1,
                        "collection_event_name": "Baseline",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 1')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP08T3.00",
                        "box_number": "2",
                        "sample_location": "A1",
                        "type_of_sample": "Stool",
                        "sample_collection_procedure": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "processed_sample_derivative": "cfDNA",
                        "sample_derivative_volume": 1.0,
                        "sample_derivative_volume_units": "Microliters",
                        "sample_derivative_concentration": 1.0,
                        "sample_derivative_concentration_units": "Nanogram per Microliter",
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                        "comments": "Comment",
                    },
                    {
                        "shipping_entry_number": 2,
                        "collection_event_name": "Pre_Day_1_Cycle_2",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 2')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP08T4.00",
                        "box_number": "2",
                        "sample_location": "A2",
                        "type_of_sample": "Stool",
                        "sample_collection_procedure": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "processed_sample_derivative": "cfDNA",
                        "sample_derivative_volume": 2.0,
                        "sample_derivative_volume_units": "Milliliters",
                        "sample_derivative_concentration": 2.0,
                        "sample_derivative_concentration_units": "Milligram per Milliliter",
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                        "comments": "No comment",
                    },
                    {
                        "shipping_entry_number": 3,
                        "collection_event_name": "Pre_Day_1_Cycle_2",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 3')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP08T5.00",
                        "box_number": "2",
                        "sample_location": "A3",
                        "type_of_sample": "Stool",
                        "sample_collection_procedure": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 2.0,
                        "processed_sample_derivative": "cfDNA",
                        "sample_derivative_volume": 3.0,
                        "sample_derivative_volume_units": "Microliters",
                        "sample_derivative_concentration": 3.0,
                        "sample_derivative_concentration_units": "Micrograms per Microliter",
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    },
                ],
                "cohort_name": "Arm_A",
                "participant_id": "test_encrypted('TTTPP8')",
                "cimac_participant_id": "CTTTP08",
            },
            {
                "samples": [
                    {
                        "collection_event_name": "Baseline",
                        "fixation_stabilization_type": "Formalin Fixation",
                        "parent_sample_id": "test_encrypted('TRIALGROUP 4')",
                        "processed_sample_id": "test_encrypted('BIOBANK 1')",
                        "cimac_id": "CTTTP09T1.00",
                        "box_number": "2",
                        "sample_location": "A4",
                        "type_of_sample": "Stool",
                        "sample_collection_procedure": "Other",
                        "processed_sample_type": "Bone Marrow Film",
                        "processed_sample_quantity": 1.0,
                        "processed_sample_derivative": "cfDNA",
                        "sample_derivative_volume": 4.0,
                        "sample_derivative_volume_units": "Milliliters",
                        "sample_derivative_concentration": 4.0,
                        "sample_derivative_concentration_units": "Cells per Vial",
                        "material_used": 1.0,
                        "material_used_units": "Milliliters",
                        "material_remaining": 0.0,
                        "material_remaining_units": "Milliliters",
                        "material_storage_condition": "Ambient",
                        "quality_of_sample": "QC Pass",
                        "sample_replacement": "Replacement Not Requested",
                        "residual_sample_use": "Not Reported",
                    }
                ],
                "cohort_name": "Arm_Z",
                "participant_id": "test_encrypted('TTTPP901')",
                "cimac_participant_id": "CTTTP09",
            },
        ],
    }
    upload_entries = []
    base_trial = get_test_trial()

    target_trial = copy_dict_with_branch(
        base_trial, prismify_patch, ["participants", "shipments"]
    )

    return PrismTestData(
        upload_type,
        prismify_args,
        prismify_patch,
        upload_entries,
        base_trial,
        target_trial,
    )


missing = set(SUPPORTED_MANIFESTS).difference(
    [f.__name__ for f in manifest_data_generators]
)
assert not missing, f"Missing manifest test data generators for {missing}"
